# authentic2_auth_fedict - Fedict authentication for Authentic
# Copyright (C) 2016  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
import requests

from django.conf import settings
from django.contrib.auth import get_user_model

import lasso

from mellon.adapters import DefaultAdapter
from authentic2.models import Attribute

class AuthenticAdapter(DefaultAdapter):
    def lookup_user(self, idp, saml_attributes):
        if 'email' in saml_attributes:
            # XXX: remove email from received attributes for now, this
            # facilitates emulating Fedict IdP with another authentic.
            del saml_attributes['email']
        # replace SAML name id content by fedict specific "fedid"
        saml_attributes['name_id_content_orig'] = saml_attributes['name_id_content']
        saml_attributes['name_id_content'] = saml_attributes['urn:be:fedict:iam:attr:fedid'][0]
        saml_attributes['name_id_format'] = lasso.SAML2_NAME_IDENTIFIER_FORMAT_UNSPECIFIED
        return super(AuthenticAdapter, self).lookup_user(idp, saml_attributes)

    def create_user(self, user_class):
        return user_class.objects.create()

    def provision_from_nrn(self, user, nrn):
        if not hasattr(settings, 'RRN_POP_SERVICE_URL'):
            return
        logger = logging.getLogger(__name__)
        try:
            response = requests.get(settings.RRN_POP_SERVICE_URL + nrn,
                    verify=False, timeout=5)
        except requests.exceptions.RequestException as e:
            logger.error('error connecting to rrn pop service (%r)', e)
            return
        if response.status_code != 200:
            logger.error('wrong status code from rrn pop service (%s)',
                    response.status_code)
            return
        attributes = response.json()
        if attributes.get('rue'):
            # fix street name
            attributes['rue'] = attributes['rue'].split('(')[0]

        attribute_mapping = [
            ('commune', 'city'),
            ('rue', 'street'),
            ('rue', 'address'), # alternative attribute name
            ('numero', 'num_house'),
            ('codePostal', 'zipcode'),
            ('boite', 'num_box'),
        ]
        for nrn_attribute, user_attribute in attribute_mapping:
            try:
                Attribute.objects.get(name=user_attribute).set_value(user,
                        attributes.get(nrn_attribute),
                        verified=True)
            except Attribute.DoesNotExist:
                pass

        user.first_name = attributes.get('prenom') or ''
        user.last_name = attributes.get('nom') or ''

    def provision_attribute(self, user, idp, saml_attributes):
        super(AuthenticAdapter, self).provision_attribute(user, idp, saml_attributes)
        if not user.email:
            # make sure the account is not usable for now
            user.is_active = False
            user.save()

        nrn = saml_attributes['egovNRN'][0]
        for attr_name in ('niss', 'nrn'):
            try:
                Attribute.objects.get(name=attr_name).set_value(user, nrn,
                        verified=True)
            except Attribute.DoesNotExist:
                pass
        if nrn[:6] == '0000001': # unknown date
            birthdate = ''
        else:
            # TODO: handle post y2k birthdates
            birthdate = '%s/%s/19%s' % (nrn[4:6], nrn[2:4], nrn[:2])
        Attribute.objects.get(name='birthdate').set_value(user, birthdate,
                verified=True)
        if int(nrn[6:9]) % 2:
            title = 'Monsieur'
        else:
            title = 'Madame'
        Attribute.objects.get(name='title').set_value(user, title,
                verified=True)

        if nrn:
            self.provision_from_nrn(user, nrn)

        user.save()
